import std.string;
import std.process;

import d2substs;
import file;
import utils;
import global_data;


class Config
{
    static Config getInstance()
    {
        if(_instance is null) _instance = new Config;
        return _instance;
    }


    string[] getKeys() { return ConfigPairs.keys.sort; }


    string getValue(string key)
    {
	    if (!key.isIn(ConfigPairs.keys))
    		    throwErrorMessage(format("Configuration Error! Key '%s' not found in config file.", key));
        return ConfigPairs[key];
    }


    void setValue(string key, string value)
    {
        ConfigPairs[key] = value;
    }


    private:
        string[string] ConfigPairs;
        static Config _instance;
        string LC_MESSAGES, LANG;


        this()
        {
            version(devel) //For using when developing
            {
                ConfigPairs["env_HOME"] = "/tmp/fake_home";

                if (!std.file.exists(ConfigPairs["env_HOME"]))
                    std.file.mkdir(ConfigPairs["env_HOME"]);

                ConfigPairs["env_XDG_CONFIG_HOME"] = ConfigPairs["env_HOME"] ~ "/.config";
            }
            else
            {
                ConfigPairs["env_HOME"] = d2substs.getenv("HOME");

                ConfigPairs["env_XDG_CONFIG_HOME"] = d2substs.getenv("XDG_CONFIG_HOME");

                if (ConfigPairs["env_XDG_CONFIG_HOME"] == "")
                    ConfigPairs["env_XDG_CONFIG_HOME"] = ConfigPairs["env_HOME"] ~ "/.config";
            }

            ConfigPairs["env_XDG_CONFIG_DIRS"] = std.string.split(d2substs.getenv("XDG_CONFIG_DIRS"), ":")[0];

            ConfigPairs["env_XDG_DATA_DIRS"] = d2substs.getenv("XDG_DATA_DIRS") ~ ":" ~ d2substs.getenv("XDG_DATA_HOME");

            ConfigPairs["openbox config dir"] = ConfigPairs["env_XDG_CONFIG_HOME"] ~ "/openbox/";
            ConfigPairs["openbox menu.xml"] = ConfigPairs["openbox config dir"] ~ "menu.xml";
            ConfigPairs["openbox rc.xml"] = ConfigPairs["openbox config dir"] ~ "rc.xml";
            ConfigPairs["openbox autostart.sh"] = ConfigPairs["openbox config dir"] ~ "autostart.sh";

            ConfigPairs["obmenugen share dir"] = "/usr/share/obmenugen/";
            ConfigPairs["obmenugen config dir"] = ConfigPairs["env_XDG_CONFIG_HOME"] ~ "/obmenugen/";
            ConfigPairs["obmenugen config file"] = ConfigPairs["obmenugen config dir"] ~ "config";
            ConfigPairs["obmenugen schema file"] = ConfigPairs["obmenugen config dir"] ~ "schema";
            ConfigPairs["obmenugen exclusions file"] = ConfigPairs["obmenugen config dir"] ~ "exclusions";


            LC_MESSAGES = d2substs.getenv("LC_MESSAGES");
            LANG        = d2substs.getenv("LANG");

            if(LC_MESSAGES) ConfigPairs["env_LANG"] = LC_MESSAGES;
            else if(LANG)   ConfigPairs["env_LANG"] = LANG;
            else            ConfigPairs["env_LANG"] = "en_US.UTF-8";

            ConfigPairs["locales long"] = std.string.split(ConfigPairs["env_LANG"], ".")[0];
            ConfigPairs["locales short"] = std.string.split(ConfigPairs["locales long"], "_")[0];

            guaranteeConfigFilesExists();

            /* Default configuration */
            ConfigPairs["terminal"] = "xterm";
            ConfigPairs["editor"] = "xterm -e nano";
            ConfigPairs["filemanager"] = "thunar";
            ConfigPairs["webbrowser"] = "firefox";
            ConfigPairs["instant messaging"] = "pidgin";
            ConfigPairs["run command"] = "gmrun";
            ConfigPairs["lock command"] = "xscreensaver-command -lock";
            ConfigPairs["exit command"] = "none";
            ConfigPairs["exclude onlyshowin"] = "kde,gnome";

            parseConfigFile();
        }


        void parseConfigFile()
        {
            foreach(uint i, line; File.Load(ConfigPairs["obmenugen config file"]).Lines())
            {
                if (line.isComment()) continue;

                string[] _line = line.split("=");

                if (_line.length < 2)
                    throwErrorMessage(format("Configuration error: Line #%d isn't a valid 'key = value' pair.", i+1));

                string[] pair = [_line[0], _line[1..$].join("=")];

                foreach(uint idx, item; pair)
                    pair[idx] = pair[idx].getClean();

                if (pair[1].length == 0) pair[1] = "none";

                ConfigPairs[pair[0]] = pair[1];
            }
        }


        void guaranteeConfigFilesExists()
        {
            string obRCxml = ConfigPairs["env_XDG_CONFIG_DIRS"] ~ "/openbox/rc.xml";

            if (!std.file.exists(obRCxml))
            {
                string msg = "Error: I can't find the file '"~obRCxml~"'\n"
                "Have you OpenBox installed?\n"
                "If you have the file in your system, simply copy it to '~/.config/openbox/'\n"
                "If you feel this is a bug, please, contact me in [neonskull at gmail.com]'";
                throwErrorMessage(msg);
            }

            if (!std.file.exists(ConfigPairs["env_XDG_CONFIG_HOME"]))
                std.file.mkdir(ConfigPairs["env_XDG_CONFIG_HOME"]);

            if (!std.file.exists(ConfigPairs["openbox config dir"]))
                std.file.mkdir(ConfigPairs["openbox config dir"]);

            if (!(std.file.exists(ConfigPairs["obmenugen config dir"])))
                std.file.mkdir(ConfigPairs["obmenugen config dir"]);

            if (!std.file.exists(ConfigPairs["openbox autostart.sh"]))
            {
                File.Write(ConfigPairs["openbox autostart.sh"], "#!/bin/bash\n");
                string cmd = std.string.format("chmod 0744 %s", ConfigPairs["openbox autostart.sh"]);
                system(cmd);
            }

            if (!std.file.exists(ConfigPairs["openbox rc.xml"]))
                std.file.copy(obRCxml, ConfigPairs["openbox rc.xml"]);

            if (!std.file.exists(ConfigPairs["obmenugen config file"]))
            {
                string oldCfgFile = ConfigPairs["obmenugen config dir"] ~ "obmenugen.cfg";
                if (std.file.exists(oldCfgFile))
                {
                    std.file.copy(oldCfgFile, ConfigPairs["obmenugen config file"]);
                    std.file.remove(oldCfgFile);
                }
                else
                    File.Write(ConfigPairs["obmenugen config file"], DEFAULT_OBMENUGEN_CFG_FILE);

            }
            if (!std.file.exists(ConfigPairs["obmenugen schema file"]))
            {
                string oldSchemaFile = ConfigPairs["obmenugen config dir"] ~ "obmenugen.schema";
                if (std.file.exists(oldSchemaFile))
                {
                    std.file.copy(oldSchemaFile, ConfigPairs["obmenugen schema file"]);
                    std.file.remove(oldSchemaFile);
                }
                else
                    File.Write(ConfigPairs["obmenugen schema file"], DEFAULT_OBMENUGEN_SCHEMA_FILE);
            }
            if (!std.file.exists(ConfigPairs["obmenugen exclusions file"]))
                File.Write(ConfigPairs["obmenugen exclusions file"], DEFAULT_OBMENUGEN_EXCLUSIONS_FILE);

        }
}

