import std.string;

import d2substs;
import utils;
import global_data;
import file;
import config;


class DesktopFileParser
{
    this()
    {
        cfg = Config.getInstance();
        initFilters();
        initExclusions();
    }
    
    string[string] parse(string filepath)
    {
        string[string] fileInfo;
        File f = File.Load(filepath);
        string[] fileLines = f.Lines();
        
        if (getNoDisplay(fileLines))
            return null;
        
        string[] onlyShowIn = getOnlyShowIn(fileLines);
        
        fileInfo["Name"] = getName(fileLines);
               
        if ((fileInfo["Name"] is null) || (isExcluded(fileInfo["Name"], onlyShowIn)))
            return null;
        
        fileInfo["Category"] = getCategory(fileLines);
        if (fileInfo["Category"] is null) return null;
        
        fileInfo["Exec"] = getExecCmd(fileLines);
        if (fileInfo["Exec"] is null) return null;
        
        fileInfo["InTerminal"] = getInTerminal(fileLines);
        
        return fileInfo;
    }


    private:
        Config cfg;
        string[string] LANG, FILTERS;
        string[][string] EXCLUSIONS;
        string[] EXCLUDE_ONLY_SHOW_IN;        


        bool isExcluded(string name, string[] only_show_in)
        {
            if (EXCLUSIONS["exact"] !is null)
                foreach(item; EXCLUSIONS["exact"])
                    if (name == item) return true;
            
            if (EXCLUSIONS["begins"] !is null)
                foreach(item; EXCLUSIONS["begins"])
                    if (name.startsWith(item)) return true;
            
            if (EXCLUSIONS["ends"] !is null)
                foreach(item; EXCLUSIONS["ends"])
                    if (name.endsWith(item)) return true;
            
            if (EXCLUSIONS["contains"] !is null)
                foreach(item; EXCLUSIONS["contains"])
                    if (find(name, item) >= 0) return true;
            
            if ((only_show_in.length > 0) && (EXCLUDE_ONLY_SHOW_IN.length > 0))
                foreach(exclusion; EXCLUDE_ONLY_SHOW_IN)
                    foreach(item; only_show_in)
                        if (item == exclusion)
                            return true;
            return false;
        }


        string getName(string[] fileLines)
        {
            foreach(line; fileLines)
                if (line.matchFilter(FILTERS["NameLangLong"]))
                    return getCleanValue(line);
            
            foreach(line; fileLines)
                if (line.matchFilter(FILTERS["NameLangShort"]))
                    return getCleanValue(line);

            foreach(line; fileLines)
                if (line.matchFilter(FILTERS["Name"]))
                    return getCleanValue(line);

            return null;
        }


        string getCategory(string[] fileLines)
        {
            foreach(line; fileLines)
            {
                if (line.matchFilter(FILTERS["Categories"]))
                {
                    line = line.replace("\n", "");
                    line = line.split("=")[1];
                    string[] categories = line.split(";");
                    foreach(cat; CATEGORIES_PRIORITY)
                    {
                        string _cat = cat[2..length];
                        foreach(category; categories)
                        {
                            if (category == _cat)
                            {
                                return tolower(_cat);
                            }
                        }
                    }
                }
            }
            return null;
        }


        string getExecCmd(string[] fileLines)
        {
            foreach(line; fileLines)
                if (line.matchFilter(FILTERS["Exec"]))
                    return getCleanValue(line, false);

            return null;
        }


        string[] getOnlyShowIn(string[] fileLines)
        {
            string[] only_show_in;
            foreach(line; fileLines)
            {
                if (line.matchFilter(FILTERS["OnlyShowIn"]))
                {
                    foreach(item; getCleanValue(line).split(";"))
                    {
                        item = getClean(item);
                        if (item.length > 0)
                            only_show_in ~= tolower(item);
                    }
                    return only_show_in;
                }
            }
            return only_show_in;
        }
        

        bool getNoDisplay(string[] fileLines)
        {
            foreach(line; fileLines)
            {
                if (line.matchFilter(FILTERS["NoDisplay"]))
                {
                    string value = tolower(getCleanValue(line));
                    if ((value == "true") || (value == "yes") || (value == "1"))
		                return true;
                }
            }
            return false;
        }

        
        string getInTerminal(string[] fileLines)
        {
            foreach(line; fileLines)
            {
                if (line.matchFilter(FILTERS["Terminal"]))
                {
                    string value = tolower(getCleanValue(line));
                    if ((value == "true") || (value == "yes") || (value == "1"))
		                return "true";
                }
            }
            return "false";
        }


        void initFilters()
        {
            LANG["Long"] = cfg.getValue("locales long");
            LANG["Short"] = cfg.getValue("locales short");
            
            FILTERS["Categories"] = "Categories=";
            FILTERS["NameLangLong"] = std.string.format("Name[%s]=", LANG["Long"]);
            FILTERS["NameLangShort"] = std.string.format("Name[%s]=", LANG["Short"]);
            FILTERS["Name"] = "Name=";
            FILTERS["Exec"] = "Exec=";
            FILTERS["OnlyShowIn"] = "OnlyShowIn=";
            FILTERS["NoDisplay"] = "NoDisplay=";
            FILTERS["Terminal"] = "Terminal=";
        }
        
        
        void initExclusions()
        {
            EXCLUSIONS["exact"] = null;
            EXCLUSIONS["begins"] = null;
            EXCLUSIONS["ends"] = null;
            EXCLUSIONS["contains"] = null;
            
            foreach(line; File.Load(cfg.getValue("obmenugen exclusions file")).Lines())
            {
                if (line.isComment()) continue;
                
                if (line.startsWith("^") && line.endsWith("$"))
                    EXCLUSIONS["exact"] ~= line[1..length-1];
                    
                else if (line.startsWith("^"))
                    EXCLUSIONS["begins"] ~= line[1..length];
                    
                else if (line.endsWith("$"))
                    EXCLUSIONS["ends"] ~= line[0..length-1];
                    
                else
                    EXCLUSIONS["contains"] ~= line;
            }
            
            foreach(item; cfg.getValue("exclude onlyshowin").split(","))
            {
                item = getClean(item);
                if (item.length > 0)
                    EXCLUDE_ONLY_SHOW_IN ~= item;
            }
        }
}

