import std.string;
import std.file;
import std.path;
import std.regexp;
import std.stdio;

import d2substs;
import i18n_common;


string VERSION = "0.1";


class I18n
{
    static I18n getInstance()
    {
        if(_instance is null) _instance = new I18n();
        return _instance;
    }


    void translation(string text_domain, string[] i18n_dirs = null, string[] locales = null)
    {
        TEXTDOMAIN = text_domain;
        if (i18n_dirs != null) I18N_DIRS = i18n_dirs;
        if (locales != null)
        {
            LOCALES = [];
            foreach(locale; locales)
                LOCALES ~= locale;
        }
    }


    void load()
    {
        if (TEXTDOMAIN == null)
            throw new I18nException("Error!!! Translation domain must be setted!");
        init_messages();
    }


    string get_version() { return "i18n class for D v" ~ VERSION; }


    string gettext(string _msg)
    {
        string key = _msg;
        if ( ((key in MESSAGES) != null) && MESSAGES[key].translation )
    	    return MESSAGES[key].translation;
	    return _msg;
    }


    debug
    {
        string get_config()
        {
            string MSGs;

            foreach(key; MESSAGES.keys)
                MSGs ~= format("    id '%s': str '%s'\n",
                               escape(key.replace("\n", "\\n")),
                               escape(MESSAGES[key].translation.replace("\n", "\\n")));

            return format("I18N DEBUG:\n  LOCALES: %s,\n  I18N_DIRS: %s,"
                          "\n  DOMAIN: %s,\n  FILE: %s,\n  MESSAGES:\n%s",
                          LOCALES, I18N_DIRS, TEXTDOMAIN, FILE, MSGs);
        }
    }


private:
    static I18n _instance;
    string TEXTDOMAIN = null;
    string[] LOCALES;
    string[] I18N_DIRS = ["/usr/share/locale"];
    TranslationInfo[string] MESSAGES;
    debug {string FILE;}


    this() { init_locales(); }


    void init_locales()
    {
        string LC_MESSAGES = d2substs.getenv("LC_MESSAGES");
        string LANG        = d2substs.getenv("LANG");

        if(LC_MESSAGES) LOCALES ~= LC_MESSAGES;
        else if(LANG)   LOCALES ~= LANG;
        else            LOCALES ~= "en_US.UTF-8";

        LOCALES ~= std.string.split(LOCALES[0], ".")[0];
        LOCALES ~= std.string.split(LOCALES[1], "_")[0];
    }


    void init_messages()
    {
        string filePath = get_messages_file_path();
        debug {FILE = filePath;}

        if (filePath != null)
        {
            PoFileParser poParser = new PoFileParser(filePath);
            MESSAGES = poParser.getParsed();
        }
        else
        {
            std.stdio.fwritefln(stderr, "i18n msg (not even a warning): No translations loaded for your current locale.");
        }
    }


    string get_messages_file_path()
    {
        string filePath;

        foreach(dir; I18N_DIRS)
            foreach(locale; LOCALES)
            {
                filePath = std.string.format("%s%s%s%s%s%s%s.po", dir, sep, locale, sep, "LC_MESSAGES", sep, TEXTDOMAIN);
                if (std.file.exists(filePath))
                    return filePath;
            }
        return null;
    }


    class I18nException : Exception { this(string _msg) { super(_msg); } }
}


string _(string _msg)
{
    I18n i18n = I18n.getInstance();
    return i18n.gettext(_msg);
}


