import std.stdio;
import std.string;

import d2substs;
import help;
import global_data;
import utils;
import file;
import config;
import desktopfile;
import openbox_mg;
import i18n;


const string VERSION =  "0.5";
const string REVISION = "72";

/*
 * Command Line Flags
 */
string category;
string action;


class CmdLineException : Exception { this(string _msg) { super(_msg); } }
class CategoryException : Exception { this(string _msg) { super(_msg); } }


void parseCmdLine(string[] argv)
{
    if (argv.length > 2)
        throw new CmdLineException("Only one parameter!");
    
    if (argv.length == 2)
    {
        string arg = argv[1];
        switch (arg)
        {
            case "-h":
            case "--help":
                action = "showHelp";
                break;
            case "-s":
            case "--static":
                action = "staticMenuXML";
                break;
            case "-p":
                action = "pipeMenuXML";
                break;
            case "--pipe":
                action = "pipeMenu";
                break;
            default:
                if (arg.startsWith("--category"))
                {
                    action = "category";
                    category = std.string.split(arg, "=")[1];
                }
                else
                {
                    string msg = std.string.format("Unknown parameter: %s", arg);
                    throw new CmdLineException(msg);
                }
        }
    }
    else
        action = "pipeMenuXML";
}

int main(string[] argv)
{
    try
    {
        Config cfg = Config.getInstance();
        
        parseCmdLine(argv);
        
        if (action == "showHelp")
        {
            Help help = new Help(VERSION, REVISION);
            writefln(help.help());
        }
        
        else
        {
            I18n i18n = I18n.getInstance();
            i18n.translation("obmenugen",
                             [cfg.getValue("obmenugen config dir") ~ "/translations/",
                              cfg.getValue("obmenugen share dir") ~ "/translations/"],
                             [cfg.getValue("env_LANG"),
                              cfg.getValue("locales long"),
                              cfg.getValue("locales short")]);
            i18n.load();
            
            DesktopFileParser dfParser = new DesktopFileParser();
        
            string[string][string] infoList;
            
            string[] DOT_DESKTOP_LOCATIONS = std.string.split(cfg.getValue("env_XDG_DATA_DIRS"), ":");
            foreach(file; getDotDesktopFiles(DOT_DESKTOP_LOCATIONS))
            {
                string[string] info = dfParser.parse(file);
                                
                if (info !is null)
                    infoList[info["Name"]~info["Exec"]] = info;
            }
            
            string menu_type;
            if (action == "category")
            {
                if (category.isIn(CATEGORIES))
                {
                    menu_type = "category";
                    cfg.setValue("category", category);
                }
                else
                    throw new CategoryException(category);
            }
            else if (action == "staticMenuXML")
            {
                std.stdio.fwritefln(stderr, "Information (Core): Generating Static 'menu.xml'.");
                menu_type = "static menu.xml";
            }
    	    else if (action == "pipeMenuXML")
    	    {
    	        std.stdio.fwritefln(stderr, "Information (Core): Setting 'menu.xml' for dinamic menus (auto-updating menus).");
    	        menu_type = "pipe menu.xml";
    	    }
    	    else if (action == "pipeMenu")
    	    {
    	        menu_type = "pipe";
    	    }
    	    
    	    
    	    cfg.setValue("menu type", menu_type);
            OpenBoxMenuGenerator mg = new OpenBoxMenuGenerator(infoList, argv);
            
            
    	    string menu = mg.getMenu();
    	    switch (menu_type)
    	    {
    	        case "category":
    	        case "pipe":
        	        writefln(menu);
        	        break;
    	        default:
        	        file.File.Write(cfg.getValue("openbox menu.xml"), menu);
    	    }
        }
    }
    catch (Exception e)
    {
        if( (startsWith(e.msg, "Unrecognized option")) ||
            (e.classinfo.name == "obmenugen.CmdLineException") ||
            (e.classinfo.name == "obmenugen.CategoryException") )
        {
            string msg;
            if(e.classinfo.name == "obmenugen.CategoryException")
                msg = format(_("Error! Unrecognized category: %s"), e.msg);
            else if ( (e.classinfo.name == "obmenugen.CmdLineException") ||
                      (e.classinfo.name == "obmenugen.CategoryException") )
            {
                string _msg;
                if (startsWith(e.msg, "Unrecognized option"))
                    _msg = split(e.msg)[2];
                else
                    _msg = e.msg;
                msg = format(_("Error! Unrecognized option: %s"), _msg);
            }
            else { msg = e.msg; }

            std.stdio.fwritefln(stderr, msg);
            Help help = new Help(VERSION, REVISION);
            std.stdio.fwritefln(stderr, help.usage());
        }
        return 1;
    }
    return 0;
}

