import std.string;
import std.path;

import d2substs;
import config;
import file;
import utils;
import global_data;
import i18n;


class OpenBoxMenuGenerator
{
    this(string[string][string] _infoList, string[] _argv)
    {
        infoList = _infoList;
        keysList = infoList.keys.sort;
        cfg = Config.getInstance();
        argv = _argv;
        
        initCategoriesInfo();
    }
    
    
    string getMenu()
    {
        menu_type = cfg.getValue("menu type");
        return theMenu();
    }
    
    
    private:
        string[string][string] infoList;
        Config cfg;
        string[] argv;
        string[] keysList;
        string menu_type;
        
        struct CategoryInfo { string id; string[] categories; string label; };
        CategoryInfo[string] CategoriesInfo;
                
        string SUBMENU = "    <menu id=\"%s\" label=\"%s\">\n%s    </menu>\n";
        string MENU_ITEM = "        <item label=\"%s\"><action name=\"Execute\"><execute>%s</execute></action></item>\n";
        

        string theMenu()
        {
            string menu;
            
            menu ~= getHeader();
            
            if (menu_type == "category")
                menu ~= getCategoryPipeMenu();
            else if (menu_type != "pipe menu.xml")
                menu ~= parseSchema();
            
            menu ~= getFooter();
            
            return menu;
        }

        
        string parseSchema()
        {
            string menu;
            foreach(uint i, line; File.Load(cfg.getValue("obmenugen schema file")).Lines())
            {
                if (line.isComment()) continue;
            
                string[] pair = line.split(":");
                                
                if (pair.length < 2)
                    throwErrorMessage(format("Schema error: Line #%d isn't a valid 'type:options' pair.", i+1));
                    
                string type = pair[0].getClean();
                string options = pair[1].getClean();
                    
                switch (type)
                {
                    case "item":
                        menu ~= getItem(options);
                        break;
                    
                    case "submenu":
                        if (options == "windowsanddesktops")
                            menu ~= getWindowsAndDesktopsMenu();
                        else if (options == "openbox")
                            menu ~= getOpenBoxMenu();
                        break;
                    
                    case "cat":
                        menu ~= getCategoryMenu(options);
                        break;
                        
                    case "sep":
                        menu ~= separator(options);
                        break;
                        
                    case "file":
                        menu ~= getFile(options);
                        break;
                        
                    case "raw":
                        menu ~= getRawLine(options);
                        break;
                        
                    default:
                        throwErrorMessage(format("Unknown 'type' definition: %s\n", type));
                }
            }
            return menu;
        }
        
        
        string getItem(string itemID)
        {
            string item = "";
            switch (itemID)
            {
                case "terminal":
                    if (cfg.getValue("terminal") == "none")
                        throwErrorMessage(_("Configuration key 'terminal' MUST be setted."));
                    item = format(MENU_ITEM[4..length], _("Terminal"), cfg.getValue("terminal"));
                    break;
                
                case "editor":
                    if (cfg.getValue("editor") == "none")
                        throwErrorMessage(_("Configuration key 'editor' MUST be setted."));
                    item = format(MENU_ITEM[4..length], _("Editor"), cfg.getValue("editor"));
                    break;
                
                case "filemanager":
                    if (cfg.getValue("filemanager") != "none")
                        item = format(MENU_ITEM[4..length], _("File Manager"), cfg.getValue("filemanager"));
                    break;
                
                case "webbrowser":
                    if (cfg.getValue("webbrowser") != "none")
                        item = format(MENU_ITEM[4..length], _("Web Browser"), cfg.getValue("webbrowser"));
                    break;
                
                case "instantmessaging":
                    if (cfg.getValue("instant messaging") != "none")
                        item = format(MENU_ITEM[4..length], _("Instant Messaging"), cfg.getValue("instant messaging"));
                    break;
                
                case "runcommand":
                    if (cfg.getValue("run command") != "none")
                        item = format(MENU_ITEM[4..length], _("Run Command"), cfg.getValue("run command"));
                    break;
                
                case "lock":
                    if (cfg.getValue("lock command") != "none")
                        item = format(MENU_ITEM[4..length], _("Lock Screen"), cfg.getValue("lock command"));
                    break;
                
                case "exit":
                    if (cfg.getValue("exit command") != "none")
                        item = format(MENU_ITEM, _("Exit"), cfg.getValue("exit command"));
                    else
                        item = format("    <item label=\"%s\"><action name=\"Exit\" /></item>\n", _("Exit"));
                    break;
                
                default:
                    throwErrorMessage(format("Unknown 'item' definition: %s\n", itemID));
            }
            
            return item;
        }
        
        
        string getFile(string filename)
        {
            string filepath = std.path.join(cfg.getValue("obmenugen config dir"), filename);
            
            if (!std.file.exists(filepath))
                throwErrorMessage(format("%s '%s'\n", _("Error! I can't find the file"), filepath));
            return File.Load(filepath).Content();
        }


        string getRawLine(string line)
        {
            return line ~ newline;
        }


        string getCategoryMenu(string category)
        {
            string menu;
            CategoryInfo catInfo = CategoriesInfo[category];
            
            string[string] tmpInfo;
                        
            foreach(key; keysList)
            {
                tmpInfo = infoList[key];
                foreach(cat; catInfo.categories)
                {
                    if (tmpInfo["Category"] == cat)
                    {
                        menu ~= menuitem(tmpInfo["Name"], tmpInfo["Exec"], tmpInfo["InTerminal"]);
                    }
                }
            }
            
            if (menu == "") return menu;
            
            return submenu(catInfo.label, catInfo.label, menu);
        }
        

        string getCategoryPipeMenu()
        {
            string category = cfg.getValue("category");
            string categoryMenu = getCategoryMenu(category);
            if(categoryMenu.length != 0)
                categoryMenu = std.string.join(splitlines(getCategoryMenu(category))[1..length-1], "\n") ~ "\n";
            return categoryMenu;
        }

        
        string getWindowsAndDesktopsMenu()
        {
            return  separator() ~ "    <menu id=\"client-list-combined-menu\" />\n";
        }
        
        
        string getOpenBoxMenu()
        {
            string menu = format("        <item label=\"%s\"><action name=\"Reconfigure\" /></item>\n", _("Reconfigure Openbox"));
    
            foreach(info; infoList)
            {
                if (info["Exec"].startsWith("obconf"))
                {
                    menu ~= menuitem(info["Name"], info["Exec"], info["InTerminal"]);
                    break;
                }
            }
            
            menu ~= format(MENU_ITEM, _("Configure Autostarted Applications"),
                                      format("%s %s", cfg.getValue("editor"), cfg.getValue("openbox autostart.sh")));
    
            menu ~= format(MENU_ITEM, _("Edit rc.xml"),
                                      format("%s %s", cfg.getValue("editor"), cfg.getValue("openbox rc.xml")));
                
            menu ~= "    " ~ separator(_("obmenugen settings"));

            menu ~= format(MENU_ITEM, _("Edit obmenugen configuration"),
                                      format("%s %s", cfg.getValue("editor"), cfg.getValue("obmenugen config file")));

            menu ~= format(MENU_ITEM, _("Edit obmenugen exclusions"),
                                      format("%s %s", cfg.getValue("editor"), cfg.getValue("obmenugen exclusions file")));

            menu ~= format(MENU_ITEM, _("Edit obmenugen schema"),
                                      format("%s %s", cfg.getValue("editor"), cfg.getValue("obmenugen schema file")));
    
            return separator() ~ submenu(_("Openbox Settings"), _("Openbox Settings"), menu);
        }


        string getHeader()
        {
            string header = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
            "<openbox_menu xmlns=\"http://openbox.org/\" "
            "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" "
            "xsi:schemaLocation=\"http://openbox.org/ file:///usr/X11R6/share/openbox/menu.xsd\">\n";
            
            if (menu_type == "static menu.xml")
                header ~= "<menu id=\"root-menu\" label=\"openbox3\">\n";
                
            else if (menu_type == "pipe menu.xml")
            {
                string appPath = argv[0];
                if (startsWith(appPath, ".")) appPath = rel2abs(appPath);
                string command = appPath ~ " --pipe";
                if (argv.length > 2)
                {
                    string extraParams = std.string.join(argv[2..length], "");
                    command ~= " " ~ extraParams;
                }
                header ~= format("<menu id=\"root-menu\" label=\"openbox3\" execute=\"%s\" />\n", command);
            }
            else
                header = "<openbox_pipe_menu>\n";

            return header;
        }

        
        string getFooter()
        {
            string footer;
            
            if (menu_type == "static menu.xml")
                footer ~= "</menu>\n";
            
            if ( (menu_type == "static menu.xml") || (menu_type == "pipe menu.xml") )
                footer ~= "</openbox_menu>\n";
            else
                footer = "</openbox_pipe_menu>";
            
            return footer;
        }


        string separator(string label = "")
        {
            if (label != "")
                label = format("label=\"%s\"", _(label));
                
            return format("    <separator %s/>\n", label);
        }
        
        
        string menuitem(string label, string exec_cmd, string in_terminal = "false")
        {
            if (in_terminal == "true")
                exec_cmd = format("%s -e %s", cfg.getValue("terminal"), exec_cmd);
                
            return format(MENU_ITEM, label, exec_cmd);
        }
        
        
        string submenu(string id, string label, string menuContent)
        {
            return format(SUBMENU, id, label, menuContent);
        }
        
        
        void initCategoriesInfo()
        {
            foreach(category; CATEGORIES)
            {
                CategoriesInfo[category] = CategoryInfo();
                CategoriesInfo[category].label = _(std.string.capitalize(category));
            }
            
            CategoriesInfo["accessories"].categories = ["utility"];

            CategoriesInfo["graphics"].categories = ["graphics"];
            
            CategoriesInfo["audiovideo"].categories = ["audiovideo", "audio", "video"];
            CategoriesInfo["audiovideo"].label = _("Audio &#38; Video");
            
            CategoriesInfo["education"].categories = ["education"];
            
            CategoriesInfo["office"].categories = ["office"];
            
            CategoriesInfo["games"].categories = ["game"];
            CategoriesInfo["games"].label = _("Games");
            
            CategoriesInfo["network"].categories = ["network"];
            
            CategoriesInfo["development"].categories = ["development"];
            
            CategoriesInfo["settings"].categories = ["settings"];
            
            CategoriesInfo["system"].categories= ["system"];
            CategoriesInfo["system"].label = _("System Tools");
        }
}

